# Generated by Django 3.0.4 on 2020-04-20 14:59

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('testapp', '0006_test_remove_onetoone_field_part1'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='testremoveonetoonefieldmodel',
            name='b',
        ),
    ]
