# Generated by Django 5.0.1 on 2024-01-29 14:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('testapp', '0024_publisher_book'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModelWithNullableFieldsOfDifferentTypes',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('int_value', models.IntegerField(null=True)),
                ('name', models.CharField(max_length=100, null=True)),
                ('date', models.DateTimeField(null=True)),
            ],
        ),
    ]
